package com.yaoyicloud.render;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.util.IdUtil;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.deepoove.poi.policy.RenderPolicy;
import com.deepoove.poi.template.ElementTemplate;

/**
 * 抽象渲染器
 *
 */
public abstract class AbstractRender {

    /*
    * 导出文件位置
    */
    protected final String cwd;

    /*
     * Docx结果文件路径
     */
    protected String docxResultPath;

    /*
     * html结果文件路径
     */
    protected String htmlResultPath;

    /*
     * pdf结果文件路径
     */
    protected String pdfResultPath;

    public AbstractRender(String cwd) {
        this.cwd = cwd;
    }

    /**
     * Docx 渲染
     *

     * @param templateFileContent 模板内容
     * @return 本地文件目录
     * @throws IOException
     */
    public final String renderDocx(Map<String, Object> dataMap, byte[] templateFileContent,
        ConfigureBuilder builder, String relationId) throws IOException {

        // 注： 报告模板的模板变量按照json序列化的结果命名
        String basicPath = this.getBasicPath();
        String reportImagePath = this.getReportImagePath();
        String label = relationId + "_" + IdUtil.fastSimpleUUID();
        //word导出位置
        String reportTempWordFile = basicPath + "/" + cwd + "/" + label + ".docx";
        // 新增：创建文件夹
        File parentDir = new File(basicPath + "/" + cwd);
        if (!parentDir.exists()) {
            boolean created = parentDir.mkdirs();
            if (!created) {
                throw new IOException("无法创建文件保存目录");
            }
        }

        Configure config = builder.build();
        XWPFTemplate template =
            XWPFTemplate.compile(new ByteArrayInputStream(templateFileContent), config).render(dataMap);
        template.writeToFile(reportTempWordFile);
        template.close();

        this.docxResultPath = reportTempWordFile;
        return this.docxResultPath;
    }

    /**
     * docx转换为pdf
     *
     * @return 本地文件目录
     * @throws IOException
     */
    public final String fromDocxToPdf() throws IOException {
        // 转换 docxResultPath 为 html
        // 加工html
        // 把 html 文件转成 pdf
        fromDocxToHtml();
        this.pdfResultPath = fromHtmlToPdf();
        return this.pdfResultPath;
    }

    /**
     * docx转换为Html
     *
     * @return 本地文件目录
     * @throws IOException
     */
    public final String fromDocxToHtml() throws IOException {
        // 转换 docxResultPath 为 html
        // 加工html
        this.htmlResultPath = cwd + "/1.html";
        return this.htmlResultPath;
    }

    /**
     * html转换为pdf
     *
     * @return 本地文件目录
     * @throws IOException
     */
    public final String fromHtmlToPdf() throws IOException {
        // 转换 htmlResultPath 为 pdf
        this.pdfResultPath = cwd + "/1.pdf";
        return this.pdfResultPath;
    }
    protected abstract String getBasicPath() throws IOException;
    protected abstract String getReportImagePath();
    protected RenderPolicy indicatorsRenderPolicy() {
        return new LoopRowTableRenderPolicy() {
            @Override
            public void render(ElementTemplate eleTemplate, Object data, XWPFTemplate template) {
                // 处理数据中的null值
                Object processedData = processData(data);
                // 调用父类渲染处理后的数据
                super.render(eleTemplate, processedData, template);
            }

            private Object processData(Object data) {
                if (data == null) {
                    return null;
                }

                if (data instanceof List) {
                    // 处理List类型数据
                    return ((List<?>) data).stream()
                            .map(this::processItem)
                            .collect(Collectors.toList());
                }

                return data;
            }

            private Object processItem(Object item) {
                if (item == null) {
                    return "-";
                }

                // 如果元素是Map，处理Map中的null值
                if (item instanceof Map) {
                    Map<?, ?> map = (Map<?, ?>) item;
                    return map.entrySet().stream()
                            .collect(Collectors.toMap(
                                    Map.Entry::getKey,
                                    e -> e.getValue() == null ? "-" : e.getValue()
                            ));
                }

                return item;
            }
        };
    }
}
