package com.yaoyicloud.service.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Map;


import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.constant.enums.ReportType;
import com.yaoyicloud.render.AntiBriberyRender;
import com.yaoyicloud.render.BasicInfoRender;
import com.yaoyicloud.service.ReportService;
import com.yaoyicloud.template.AcademicAssociationReport;
import com.yaoyicloud.template.FoundationReport;
import com.yaoyicloud.template.PlatformCompanyReport;

import org.springframework.stereotype.Service;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 * 报告服务
 *
 * @author snows
 * @date 2024/10/12
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ReportServiceImpl implements ReportService {
    private final PlatformCompanyReport platformCompanyReport;
    private final AcademicAssociationReport academicAssociationReport;
    private final FoundationReport foundationReport;
    private final FilerepoProperties filerepoProperties;


    @SuppressWarnings("checkstyle:ReturnCount")
    @Override
    public String createPlusVersionCheckReport(
            ReportType reportType, String data, byte[] templateBytes, String outputBasePath, Long relationId, HttpServletRequest request, ModuleType moduleType)
        throws Exception {

        // 1. 将字节流模版写入临时文件
        File tempTemplateFile = File.createTempFile("template_", ".docx");
        try (FileOutputStream fos = new FileOutputStream(tempTemplateFile)) {
            fos.write(templateBytes);
        }
        Map<String, Object> processedData = null;
        String sessionId = request.getSession().getId();
        switch (reportType) {
            case PLATFORM_COMPANY:
                switch (moduleType) {
                    case  AntiBribery:
                         AntiBriberyRender antiBriberyRender = new AntiBriberyRender("temp", filerepoProperties);
                       // AntiBriberyRender antiBriberyRender = new AntiBriberyRender(sessionId, filerepoProperties);
                         antiBriberyRender.renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                         break;
                    case BasicInfo:
                     //   BasicInfoRender basicInfoRender = new BasicInfoRender("../temp/", filerepoProperties);
                        BasicInfoRender basicInfoRender = new BasicInfoRender("temp", filerepoProperties);
                      return   basicInfoRender.renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                    default:
                        throw new UnsupportedOperationException("Unsupported module type: " + moduleType);

                }
//                 processedData = platformCompanyReport.processData(data);
//                return platformCompanyReport.exportReport(
//                        processedData,
//                    tempTemplateFile.getAbsolutePath(),
//                    outputBasePath,
//                    relationId);
//            case ACADEMIC_ASSOCIATIONS:
//                processedData = academicAssociationReport.processData(data);
//                return academicAssociationReport.exportReport(
//                        processedData,
//                        tempTemplateFile.getAbsolutePath(),
//                        outputBasePath,
//                        relationId);
//            case FOUNDATION:
//                processedData = foundationReport.processData(data);
//                return foundationReport.exportReport(
//                        processedData,
//                        tempTemplateFile.getAbsolutePath(),
//                        outputBasePath,
//                        relationId);
            case PROMOTION:
                switch (moduleType) {
                    case  AntiBribery:
                        // AntiBriberyRender antiBriberyRender = new AntiBriberyRender("../temp/");
                        AntiBriberyRender antiBriberyRender = new AntiBriberyRender(sessionId, filerepoProperties);
                        antiBriberyRender.renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                        break;
                    case BasicInfo:
                        //   BasicInfoRender basicInfoRender = new BasicInfoRender("../temp/", filerepoProperties);
                        BasicInfoRender basicInfoRender = new BasicInfoRender(sessionId, filerepoProperties);
                   return  basicInfoRender.renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                     //   break;
                    default:
                        throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
                }
            default:
                throw new UnsupportedOperationException("Unsupported report type: " + reportType);
        }
    }


}



